<?php

declare(strict_types=1);

namespace app\models;

use yii;

use app\models\traits\SearchByEdge;

use app\models\Account;

use Exception;

/**
 * Поиск
 *
 * @see Product Поиск по товарам
 *
 * @todo Нет смысла каждый раз возвращать кнопку, надо написать проверки
 */
class Notification extends Document
{
    use SearchByEdge;

    /**
     * Сценарий для доверенного пользователя с созданием уведомления
     */
    const SCENARIO_TRUSTED_CREATE = 'create';

    /**
     * Тип уведомления: памятка
     */
    const TYPE_NOTICE = 'notice';

    /**
     * Тип уведомления: предупреждение
     */
    const TYPE_WARNING = 'warning';

    /**
     * Тип уведомления: ошибка
     */
    const TYPE_ERROR = 'error';

    /**
     * Цель для отправки уведомления
     *
     * Расшифровывается как $target
     *
     * @see SCENARIO_TRUSTED_CREATE
     */
    public string|null $account;

    /**
     * Текст уведомления
     */
    public string $text;

    /**
     * Разделитель получателей уведомлений
     */
    public static string $delimiter = ',';

    /**
     * Типы уведомлений
     */
    public array $typs = [
        'notice' => 'Уведомление',
        'warning' => 'Предупреждение'
    ];

    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return 'notification';
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'html',
                'type'
            ]
        );
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'html' => 'HTML',
                'type' => 'Тип',
                'trgt' => 'Получатели',
                'text' => 'Текст',
            ]
        );
    }

    /**
     * Правила
     */
    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    'html',
                    'required'
                ],
                [
                    'type',
                    'default',
                    'value' => 'notice'
                ]
            ]
        );
    }

    /**
     * Запись
     */
    public function write(): self|array|null
    {
        return $this::_write($this->text, $this->html, $this->account, $this->type);
    }

    /**
     * Запись
     *
     * @param string $html Содержимое уведомления (HTML или текст)
     * @param bool|string|null $html Содержимое уведомления (HTML или текст)
     * @param string $account Получатель уведомления
     * @param string $type Тип уведомления
     */
    public static function _write(string $text, bool|string|null $html = false, string $account = null, string $type = self::TYPE_NOTICE): self|array|null
    {
        // Инициализация
        $model = new self;
        $account or $account = yii::$app->user->identity->_key ?? throw new Exception('Не удалось инициализировать получателя');

        if ((bool) (int) $html) {
            // Получен текст в формете HTML-кода

            $model->html = $text ?? null;
        } else {
            // Получен необработанный текст

            $text = htmlspecialchars(strip_tags($text ?? null));

            $model->html = <<<HTML
                <p class="my-2 mx-3">$text</p>
            HTML;
        }

        if ($model->save()) {
            // Уведомление записано

            // Инициализация получателей и создание ребра
            self::searchReceiverAndConnect($model, $account, $type);
        }

        return null;
    }

    /**
     * Поиск получателя
     *
     * @param self $model Уведомление
     * @param string $text Необработанный текст
     * @param string $type Тип уведомления
     */
    protected static function searchReceiverAndConnect(self $model, string $text, string $type = self::TYPE_NOTICE): AccountEdgeNotification|array|null
    {
        // Инициализация
        $return = [];

        // Конвертация
        $accounts = array_map('trim', explode(self::$delimiter, $text));

        foreach ($accounts as $account) {
            if (in_array('@all', $accounts, true)) {
                // Найден флаг обозначающий отправку всем пользователям

                // Инициализация
                $return = [];

                foreach (Account::readAll() as $account) {
                    // Перебор всех аккаунтов

                    // Запись ребра: УВЕДОМЛЕНИЕ -> АККАУНТ
                    $return[] = AccountEdgeNotification::writeSafe($model->readId(), $account->readId(), $type);
                }
            } else if (in_array('@test', $accounts, true)) {
                // Найден флаг обозначающий тестирование (отправка самому себе)

                $return[] = AccountEdgeNotification::writeSafe($model->readId(), yii::$app->user->id, $type);
            } else {
                // Найден идентификатор (подразумевается)

                if ($account = Account::searchById(Account::collectionName() . '/' . $account)) {
                    // Аккаунт найден

                    // Запись ребра: УВЕДОМЛЕНИЕ -> АККАУНТ
                    $return[] = AccountEdgeNotification::writeSafe($model->readId(), $account->readId(), $type);
                }
            }
        }

        return $return ? $return : null;
    }
}
