<?php

declare(strict_types=1);

namespace app\models;

use yii;
use yii\web\User as Account;

use app\models\traits\SearchByEdge;

use Exception;

/**
 * Заказ
 *
 * @see Account Заказчик
 * @see Supply Поставки для заказа
 */
class Order extends Document
{
    use SearchByEdge;

    /**
     * Поставки для записи
     */
    public array $supplies;

    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return 'order';
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'stts'
            ]
        );
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'stts' => 'Статус'
            ]
        );
    }

    /**
     * Правила
     */
    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    'stts',
                    'string',
                    'message' => '{attribute} должен быть строкой'
                ],
                [
                    'stts',
                    'default',
                    'value' => 'preparing'
                ]
            ]
        );
    }

    /**
     * Подключение к аккаунту
     */
    public function connect(Account $account): ?AccountEdgeOrder
    {
        // Запись ребра: АККАУНТ -> ЗАКАЗ
        return AccountEdgeOrder::write($account->id, $this->readId(), 'current') ?? throw new Exception('Не удалось инициализировать ребро: АККАУНТ -> ЗАКАЗ');
    }

    /**
     * Запись товара
     *
     * $supply = [ Supply $supply, int $amount = 1 ]
     *
     * @param Supply|array $supply Поставка
     * @param Account $trgt Заказчик
     *
     * @return int Количество записанных поставок
     *
     * @todo Создать параметр разделителя для администрации
     */
    public function writeSupply(Supply|string|array $supply, Account $trgt = null): int
    {
        // Инициализация
        $trgt ?? $trgt = yii::$app->user ?? throw new Exception('Не удалось инициализировать заказчика');

        if ($supply instanceof Supply) {
            // Передана инстанция класса поставки или второй элемент массива не является числом

            // Унификация входных данных
            $supply = [$supply->catn => 1];
        }

        if (is_null($this->_key)) {
            // Корзина не инициализирована

            // Инициализация
            if (!$this->save()) {
                // Инициализация заказа не удалась

                throw new Exception('Ошибка при записи заказа в базу данных');
            }

            // Инициализация ребра: АККАУНТ -> ЗАКАЗ
            if (!AccountEdgeOrder::write($trgt->readId(), $this->readId(), 'create')) {
                // Инициализация не удалась

                throw new Exception('Ошибка при записи ребра от аккаунта до заказа в базу данных');
            }
        }

        // Инициализация
        $amount = 0;

        foreach (is_array($supply) ? $supply : [$supply => 1] as $supply_raw => $amount_raw) {
            // Перебор поставок

            for ($i = 0; $i < $amount_raw; $i++) {
                // Создание рёбер соразмерно запросу (добавление нескольких продуктов в корзину)

                // Запись ребра: ЗАКАЗ -> ПОСТАВКА
                if (!$supply_model = Supply::searchByCatn($supply_raw) or !OrderEdgeSupply::write($this->readId(), $supply_model->readId(), 'write')) {
                    // Поставка не найдена или запись ребра не удалась

                    continue;
                } else {
                    // Ребро создано (товар подключен к заказу)

                    // Постинкрементация счётчика добавленных товаров
                    $amount++;

                    // Запись в журнал
                    $this->journal('write', ['target' => $supply_model->readId()]);
                }
            }
        }

        if ($amount === 0) {
            // Отправка уведомления
            self::notification('Неудачная попытка добавить товар в корзину');
        } else if ($amount === 1) {
            // Отправка уведомления
            self::notification('Товар ' . $supply_model->catn . ' добавлен в корзину');
        } else {
            // Отправка уведомления
            self::notification('Добавлено ' . $amount . ' товаров в корзину');
        }

        return $amount;
    }

    /**
     * Удаление поставки
     *
     * @param Supply|string|array $supply Товары
     *
     * @return int Количество удалённых рёбер
     */
    public function deleteSupply(Supply|string|array $supply): int
    {
        // Инициализация
        $amount = 0;

        if ($supply instanceof Supply) {
            // Передана инстанция класса поставки или второй элемент массива не является числом

            // Унификация входных данных
            $supply = [$supply->catn => 1];
        }

        foreach (is_array($supply) ? $supply : [$supply => 1] as $catn => $amount_raw) {
            // Перебор товаров

            if ($supply = Supply::searchByCatn($catn)) {
                foreach (OrderEdgeSupply::searchByVertex($this->readId(), $supply->readId(), limit: $amount_raw) as $edge) {
                    // Перебор рёбер до продукта (если товаров в заказе несколько)

                    // Удаление
                    $edge->delete();

                    // Запись в журнал
                    $this->journal('delete', ['target' => $supply->readId()]);

                    // Постинкрементация счётчика удалённых рёбер
                    $amount++;
                }
            }
        }

        if ($amount === 0) {
            // Отправка уведомления
            self::notification('Неудачная попытка удалить товар из корзины');
        } else if ($amount === 1) {
            // Отправка уведомления
            self::notification('Товар ' . $supply->catn . ' удалён из корзины');
        } else {
            // Отправка уведомления
            self::notification('Удалено ' . $amount . ' товаров из корзины');
        }

        return $amount;
    }

    /**
     * Поиск заказа
     */
    public static function search(Account $account = null, string $type = 'current', int $limit = 1, int $page = 1, string $select = null): self|array|null
    {
        // Инициализация
        $account or $account = yii::$app->user ?? throw new Exception('Не удалось инициализировать пользователя');

        // Генерация сдвига по запрашиваемым данным (пагинация)
        $offset = $limit * ($page - 1);

        if (strcasecmp($type, 'all') !== 0) {
            // Если не указан параметр поиска всех заказов

            $where_type = [
                'account_edge_order.type' => $type
            ];
        } else {
            $where_type = [];
        }

        $return = self::searchByEdge(
            from: 'account',
            to: 'order',
            subquery_where: [
                [
                    'account._id' => $account->id
                ],
                $where_type
            ],
            foreach: ['edge' => 'account_edge_order'],
            where: 'edge._to == order._id',
            limit: $limit,
            offset: $offset,
            sort: ['DESC'],
            select: $select,
            direction: 'INBOUND'
        );

        return $limit === 1 ? $return[0] ?? null : $return;
    }

    /**
     * Поиск содержимого заказа
     *
     * @todo В будущем возможно заказ не только поставок реализовать
     */
    public function content(int $limit = 1, int $page = 1): Supply|array|null
    {
        // Генерация сдвига по запрашиваемым данным (пагинация)
        $offset = $limit * ($page - 1);

        // Поиск рёбер: ЗАКАЗ -> ПОСТАВКА
        $supplies = Supply::searchByEdge(
            from: 'order',
            to: 'supply',
            edge: 'order_edge_supply',
            subquery_where: [
                [
                    'order._id' => $this->readId()
                ]
            ],
            foreach: ['edge' => 'order_edge_supply'],
            where: 'edge._to == supply._id',
            limit: $limit,
            offset: $offset,
            direction: 'INBOUND'
        );

        // Инициализация реестра дубликатов
        $registry = [];

        // Подсчёт и перестройка массива для очистки от дубликатов
        foreach ($supplies as $key => &$supply) {
            // Перебор поставок

            if (in_array($supply->catn, $registry)) {
                // Если данная поставка найдена в реестре

                // Удаление
                unset($supplies[$key]);

                // Пропуск итерации
                continue;
            }

            // Инициализация
            $amount = 0;

            // Повторный перебор для поиска дубликатов
            foreach ($supplies as &$supply4check) {
                if ($supply == $supply4check) {
                    // Найден дубликат

                    // Постинкрементация счётчика
                    $amount++;

                    // Запись в реестр
                    $registry[] = $supply4check->catn;
                }
            }

            // Запись количества для заказа
            $supply->amnt = $amount;
        }

        // Поиск стоимости для каждой поставки
        foreach ($supplies as $key => &$supply) {
            // Перебор поставок

            // Чтение стоимости
            $cost = $supply->readCost();

            if ($cost < 1) {
                // Если стоимость равна нулю (явная ошибка)

                // Удаление из базы данных
                $this->deleteSupply($supply->readId());

                // Удаление из списка
                unset($supplies[$key]);

                // Пропуск итерации
                continue;
            }

            // Запись цены
            $supply->cost = $cost['ЦенаЗаЕдиницу'] . ' ' . $cost['Валюта'];
        }

        return $supplies;
    }

    /**
     * Отправка уведомления
     */
    public static function notification(string $text, string|null $type = Notification::TYPE_NOTICE): Notification|array|null
    {
        // Отправка
        return Notification::_write($text, type: $type);
    }
}
